pro recon_traj

; Paul Withers, 2001.07.11
; Open University, Britain

; Called by:				N/A
; Calls:				get_n, get_dt, get_omega,
;					setup_arrays, get_entry_state,
; 					set_initial, get_altlatlon, get_acc, 
;					update_arrays, derivs, get_nmonte,
;					get_uncert, get_openchute, $
;					sigmatrajonly
; Declares common blocks:		data_block, declare_arrays
; Reads from:				acc.dat
; Writes to:				aeroacc_res.dat, vrel_res.dat, 
;					height_res.dat, lat_res.dat,  
;					lon_res.dat, t_res.dat, speed_res.dat,
;					monte_t.dat, monte_height.dat,
;					monte_lat.dat, monte_lon.dat,
;					monte_rho.dat, monte_press.dat,
;					monte_temp.dat, bestandsigma.dat
; Options:				N/A
; Purpose:				Reconstruct trajectory and 
;					information necessary to
;					reconstruct atmospheric structure

common data_block, omega, nsection, t_array, $
 aeroaccsctx_array, aeroaccscty_array, aeroaccsctz_array
; Common block used by RK4 integration routine

common declare_arrays, x_array, y_array, z_array, $
 vx_array, vy_array, vz_array, $
 height_array, lat_array, lon_array, $
 vxrel_array, vyrel_array, vzrel_array, $
 aeroacc_array, vrel_array, $
 aeroaccinex_array, aeroacciney_array, aeroaccinez_array, $
 vwind_array, g_array, axial_array, normal_array
; Common block used by setup_arrays - and other stuff
; All these arrays are discussed in setup_arrays

common marsblk
; Properties of Mars and physical constants
; Defined in marsprops

common sctblk
; Properties of spacecraft and datasets
; Defined in sctprops

forward_function get_n, get_dt, get_omega, setup_arrays, get_entry_state, $ 
 set_initial, get_altlatlon, get_acc, update_arrays, derivs, get_nmonte, $
 get_openchute, sigmatrajonly
; Various subroutines

print, 'Starting recon_traj.pro'

uncert = sctpropsstruct.uncert
nmonte_traj = sctpropsstruct.nmonte_traj
n = sctpropsstruct.n
dt = sctpropsstruct.dt
sct = sctpropsstruct.sct
r0 = sctpropsstruct.r0
omega = marspropsstruct.omega
datafile = sctpropsstruct.datafile
nsection = sctpropsstruct.nsection
outdir = sctpropsstruct.outdir

nchute = sctpropsstruct.nchute
ndetect = sctpropsstruct.ndetect


if uncert ne 0. then begin
monte_t = dblarr(nmonte_traj,n)
monte_height = monte_t
monte_lat = monte_t
monte_lon = monte_t
monte_vrel = monte_t
monte_speed = monte_t
monte_aeroacc = monte_t
monte_axial = monte_t
monte_normal = monte_t
monte_aeroaccsctx = monte_t
monte_aeroaccscty = monte_t
monte_aeroaccsctz = monte_t
endif

j=0
while j lt nmonte_traj do begin
print, 'j = ', j, ' nmonte_traj = ', nmonte_traj

t_array = dblarr(n)
aeroaccsctx_array = dblarr(n)
aeroaccscty_array = dblarr(n)
aeroaccsctz_array = dblarr(n)
saeroaccsctx_array = dblarr(n)
saeroaccscty_array = dblarr(n)
saeroaccsctz_array = dblarr(n)

openr, lun1, datafile, /get_lun
readf, lun1, t_array
readf, lun1, aeroaccsctx_array
readf, lun1, aeroaccscty_array
readf, lun1, aeroaccsctz_array
readf, lun1, saeroaccsctx_array
readf, lun1, saeroaccscty_array
readf, lun1, saeroaccsctz_array
free_lun, lun1

aeroaccsctx_array = aeroaccsctx_array + randomn(seed, n) * saeroaccsctx_array * uncert
aeroaccscty_array = aeroaccscty_array + randomn(seed, n) * saeroaccscty_array * uncert
aeroaccsctz_array = aeroaccsctz_array + randomn(seed, n) * saeroaccsctz_array * uncert

blah=0 ; adjust manually
if blah ne 0 then begin
; Adjust a-parallel using alpha...
alpha_array = dblarr(sctpropsstruct.nchute-sctpropsstruct.ndetect)
openr, lun1, 'output/atm_'+sct+'_alpha_nominal.dat', /get_lun
readf, lun1, alpha_array
junk = dblarr(n)
junk(0:n_elements(alpha_array)-1) = alpha_array(*)
alpha_array = junk

alpha_array(*) = 5.

junk = aeroaccsctz_array * cos(!pi/180. * alpha_array) ; + $
; sqrt(aeroaccsctx_array^2 + aeroaccscty_array^2 ) * sin(!pi/180. * alpha_array) 
aeroaccsctz_array = junk
endif

setup_arrays, n
; Setup many useful arrays for putting data into

; Planetary rotation rate, rad s-1
print, '(rad s-1) Rotation rate = ', omega

print, 'Finding initial conditions'
set_initial, get_entry_state(), omega*t_array(0)
; Convert the initial conditions into the inertial cartesian frame
; and pop data into the appropriate arrays

something = get_acc(t_array(0), $
 x_array(0), y_array(0), z_array(0), $
 height_array(0), lat_array(0), lon_array(0), $
 vx_array(0), vy_array(0), vz_array(0), $
 aeroaccsctx_array(0), aeroaccscty_array(0), aeroaccsctz_array(0), $
 omega)
; Pass all the necessary information to get_acc
; which calculates the total acceleration on the spacecraft
; in the inertial cartesian frame

update_arrays, 0, something
; Some of the results from get_acc, such as the relative velocity,
; are useful to track in the arrays that setup_arrays setup

print, 'Starting trajectory integration forward from entry state ', $
 'using known accelerations'
; Start integrating forward from entry state!
i = 1
while i lt n do begin
; 1 because we already know conditions at i=0
; n+1 because we will integrate one timestep beyond the last datapoint

something = get_acc(t_array(i-1), $
 x_array(i-1), y_array(i-1), z_array(i-1), $
 height_array(i-1), lat_array(i-1), lon_array(i-1), $
 vx_array(i-1), vy_array(i-1), vz_array(i-1), $
 aeroaccsctx_array(i-1), aeroaccscty_array(i-1), aeroaccsctz_array(i-1), $
 omega)
; Pass all the necessary information to get_acc
; which calculates the total acceleration on the spacecraft
; in the inertial cartesian frame

update_arrays, i, something
; Some of the results from get_acc, such as the relative velocity,
; are useful to track in the arrays that setup_arrays setup

if (i le (nsection+1)) or (i ge n-(nsection+1)) then begin
; RK4 integration routine needs estimates of position and velocity
; between data points. I have used an interpolation procedure
; to find that which needs nsection data points on either side of
; where the interpolated data will belong
; So I can't use this technique for the first or last few data points
; This also makes it easy to edit this file
; to do first-order integrations only...
new_vec = dblarr(6)
new_vec(0) = x_array(i-1) + something(0)*dt
new_vec(1) = y_array(i-1) + something(1)*dt
new_vec(2) = z_array(i-1) + something(2)*dt

new_vec(3) = vx_array(i-1) + something(3)*dt
new_vec(4) = vy_array(i-1) + something(4)*dt
new_vec(5) = vz_array(i-1) + something(5)*dt
; Simple first order integration technique
endif

if (i gt (nsection+1)) and (i lt n-(nsection+1)) then begin
new_vec = rk4([x_array(i-1), y_array(i-1), z_array(i-1), $
 vx_array(i-1), vy_array(i-1), vz_array(i-1)], $
 [vx_array(i-1), vy_array(i-1), vz_array(i-1), $
 aeroaccinex_array(i-1)+g_array(0,i-1), $
 aeroacciney_array(i-1)+g_array(1,i-1), $
 aeroaccinez_array(i-1)+g_array(2,i-1)], $
 t_array(i-1), dt, 'derivs')
; Integrate trajectory using RK4 procedure
; Since acceleration in inertial cartesian frame depends on
; position and velocity in that frame, interpolating to midway
; between data points gets very messy
endif

x_array(i) = new_vec(0)
y_array(i) = new_vec(1)
z_array(i) = new_vec(2)
vx_array(i) = new_vec(3)
vy_array(i) = new_vec(4)
vz_array(i) = new_vec(5)
; Update arrays

junk = get_altlatlon(x_array(i),y_array(i),z_array(i),omega*t_array(i))
height_array(i) = junk(0)
lat_array(i) = junk(1)
lon_array(i) = junk(2)
; Update arrays

i = i+1
endwhile

print, 'Chute state is'
print, '(s) t = ', t_array(nchute-1)
print, '(m) r = ', height_array(nchute-1)
print, '(km) z = ', (height_array(nchute-1)-r0)/1e3
print, '(degN) lat = ', lat_array(nchute-1)
print, '(degE) lon = ', lon_array(nchute-1)
print, '(m s-1) vrel = ', vrel_array(nchute-1)
print, ''
print, 'Landed/impact state is'
print, '(s) t = ', t_array(n-1)
print, '(m) r = ', height_array(n-1)
print, '(km) z = ', (height_array(n-1)-r0)/1e3
print, '(degN) lat = ', lat_array(n-1)
print, '(degE) lon = ', lon_array(n-1)
print, '(m s-1) vrel = ', vrel_array(n-1)

speed_array = sqrt(vx_array^2 + vy_array^2 + vz_array^2)
;axial_array = sqrt( aeroaccsctz_array^2 )
axial_array = aeroaccsctz_array
normal_array = sqrt( aeroaccsctx_array^2 + aeroaccscty_array^2 )

if uncert eq 0 then begin
openw, lun1, outdir + 'traj_' + sct + '_aeroacc_nominal.dat', /get_lun
printf, lun1, aeroacc_array
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_vrel_nominal.dat', /get_lun
printf, lun1, vrel_array
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_height_nominal.dat', /get_lun
printf, lun1, height_array
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_t_nominal.dat', /get_lun
printf, lun1, t_array
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_lat_nominal.dat', /get_lun
printf, lun1, lat_array
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_lon_nominal.dat', /get_lun
printf, lun1, lon_array
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_speed_nominal.dat', /get_lun
printf, lun1, speed_array
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_axial_nominal.dat', /get_lun
printf, lun1, axial_array
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_normal_nominal.dat', /get_lun
printf, lun1, normal_array
free_lun, lun1

endif



if uncert ne 0. then begin

monte_height(j,*) = height_array(*)
monte_lat(j,*) = lat_array(*)
monte_lon(j,*) = lon_array(*)
monte_t(j,*) = t_array(*)

monte_vrel(j,*) = vrel_array(*) 
monte_speed(j,*) = speed_array(*)
monte_aeroacc(j,*) = aeroacc_array(*)
monte_axial(j,*) = axial_array(*)
monte_normal(j,*) = normal_array(*)
monte_aeroaccsctx(j,*) = aeroaccsctx_array(*)
monte_aeroaccscty(j,*) = aeroaccscty_array(*)
monte_aeroaccsctz(j,*) = aeroaccsctz_array(*)

endif ; if uncert ne 0. 

j=j+1
endwhile



if uncert ne 0. then begin

openw, lun1, outdir + 'traj_' + sct + '_monte_t.dat', /get_lun
printf, lun1, monte_t
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_height.dat', /get_lun
printf, lun1, monte_height
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_lat.dat', /get_lun
printf, lun1, monte_lat
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_lon.dat', /get_lun
printf, lun1, monte_lon
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_vrel.dat', /get_lun
printf, lun1, monte_vrel
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_speed.dat', /get_lun
printf, lun1, monte_speed
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_aeroacc.dat', /get_lun
printf, lun1, monte_aeroacc
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_axial.dat', /get_lun
printf, lun1, monte_axial
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_normal.dat', /get_lun
printf, lun1, monte_normal
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_aeroaccsctx.dat', /get_lun
printf, lun1, monte_aeroaccsctx
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_aeroaccscty.dat', /get_lun
printf, lun1, monte_aeroaccscty
free_lun, lun1
openw, lun1, outdir + 'traj_' + sct + '_monte_aeroaccsctz.dat', /get_lun
printf, lun1, monte_aeroaccsctz
free_lun, lun1

best_height = dblarr(n)
best_lat = best_height
best_lon = best_height
best_t = best_height
best_vrel = best_height
best_speed = best_height
best_aeroacc = best_height
best_axial = best_height
best_normal = best_height
best_aeroaccsctx = best_height
best_aeroaccscty = best_height
best_aeroaccsctz = best_height

sigma_height = best_height
sigma_lat = best_height
sigma_lon = best_height
sigma_t = best_height
sigma_vrel = best_height
sigma_speed = best_height
sigma_aeroacc = best_height
sigma_axial = best_height
sigma_normal = best_height
sigma_aeroaccsctx = best_height
sigma_aeroaccscty = best_height
sigma_aeroaccsctz = best_height

print, 'Calculating mean and uncertainty for results'
print, 'nmonte_traj = ', nmonte_traj
i=0L
while i lt n do begin
best_height(i) = mean(monte_height(*,i))
best_lat(i) = mean(monte_lat(*,i))
best_lon(i) = mean(monte_lon(*,i))
best_t(i) = mean(monte_t(*,i))
best_vrel(i) = mean(monte_vrel(*,i))
best_speed(i) = mean(monte_speed(*,i))
best_aeroacc(i) = mean(monte_aeroacc(*,i))
best_axial(i) = mean(monte_axial(*,i))
best_normal(i) = mean(monte_normal(*,i))
best_aeroaccsctx(i) = mean(monte_aeroaccsctx(*,i))
best_aeroaccscty(i) = mean(monte_aeroaccscty(*,i))
best_aeroaccsctz(i) = mean(monte_aeroaccsctz(*,i))

junk = moment(monte_height(*,i))
sigma_height(i) = sqrt(junk(1))
junk = moment(monte_lat(*,i))
sigma_lat(i) = sqrt(junk(1))
junk = moment(monte_lon(*,i))
sigma_lon(i) = sqrt(junk(1))
junk = moment(monte_t(*,i))
sigma_t(i) = sqrt(junk(1))
junk = moment(monte_vrel(*,i))
sigma_vrel(i) = sqrt(junk(1))
junk = moment(monte_speed(*,i))
sigma_speed(i) = sqrt(junk(1))
junk = moment(monte_aeroacc(*,i))
sigma_aeroacc(i) = sqrt(junk(1))
junk = moment(monte_axial(*,i))
sigma_axial(i) = sqrt(junk(1))
junk = moment(monte_normal(*,i))
sigma_normal(i) = sqrt(junk(1))
junk = moment(monte_aeroaccsctx(*,i))
sigma_aeroaccsctx(i) = sqrt(junk(1))
junk = moment(monte_aeroaccscty(*,i))
sigma_aeroaccscty(i) = sqrt(junk(1))
junk = moment(monte_aeroaccsctz(*,i))
sigma_aeroaccsctz(i) = sqrt(junk(1))

i=i+1
endwhile

openw, lun1, outdir + 'traj_' + sct + '_aeroacc_mean.dat', /get_lun
printf, lun1, best_aeroacc
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_vrel_mean.dat', /get_lun
printf, lun1, best_vrel
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_height_mean.dat', /get_lun
printf, lun1, best_height
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_t_mean.dat', /get_lun
printf, lun1, best_t
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_lat_mean.dat', /get_lun
printf, lun1, best_lat
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_lon_mean.dat', /get_lun
printf, lun1, best_lon
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_speed_mean.dat', /get_lun
printf, lun1, best_speed
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_axial_mean.dat', /get_lun
printf, lun1, best_axial
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_normal_mean.dat', /get_lun
printf, lun1, best_normal
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_aeroaccsctx_mean.dat', /get_lun
printf, lun1, best_aeroaccsctx
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_aeroaccscty_mean.dat', /get_lun
printf, lun1, best_aeroaccscty
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_aeroaccsctz_mean.dat', /get_lun
printf, lun1, best_aeroaccsctz
free_lun, lun1



openw, lun1, outdir + 'traj_' + sct + '_aeroacc_sigma.dat', /get_lun
printf, lun1, sigma_aeroacc
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_vrel_sigma.dat', /get_lun
printf, lun1, sigma_vrel
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_height_sigma.dat', /get_lun
printf, lun1, sigma_height
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_t_sigma.dat', /get_lun
printf, lun1, sigma_t
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_lat_sigma.dat', /get_lun
printf, lun1, sigma_lat
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_lon_sigma.dat', /get_lun
printf, lun1, sigma_lon
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_speed_sigma.dat', /get_lun
printf, lun1, sigma_speed
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_axial_sigma.dat', /get_lun
printf, lun1, sigma_axial
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_normal_sigma.dat', /get_lun
printf, lun1, sigma_normal
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_aeroaccsctx_sigma.dat', /get_lun
printf, lun1, sigma_aeroaccsctx
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_aeroaccscty_sigma.dat', /get_lun
printf, lun1, sigma_aeroaccscty
free_lun, lun1

openw, lun1, outdir + 'traj_' + sct + '_aeroaccsctz_sigma.dat', /get_lun
printf, lun1, sigma_aeroaccsctz
free_lun, lun1


print, 'Chute state is'
print, '(s) t = ', best_t(nchute-1), ' +/- ', sigma_t(nchute-1)
print, '(m) r = ', best_height(nchute-1), ' +/- ', sigma_height(nchute-1)
print, '(km) z = ', (best_height(nchute-1)-r0)/1e3, ' +/- ', sigma_height(nchute-1)/1e3
print, '(degN) lat = ', best_lat(nchute-1), ' +/- ', sigma_lat(nchute-1)
print, '(degE) lon = ', best_lon(nchute-1), ' +/- ', sigma_lon(nchute-1)
print, '(m s-1) vrel = ', best_vrel(nchute-1), ' +/- ', sigma_vrel(nchute-1)
print, ''
print, 'Landed/impact state is'
print, '(s) t = ', best_t(n-1), ' +/- ', sigma_t(n-1)
print, '(m) r = ', best_height(n-1), ' +/- ', sigma_height(n-1)
print, '(km) z = ', (best_height(n-1)-r0)/1e3, ' +/- ', sigma_height(n-1)/1e3
print, '(degN) lat = ', best_lat(n-1), ' +/- ', sigma_lat(n-1)
print, '(degE) lon = ', best_lon(n-1), ' +/- ', sigma_lon(n-1)
print, '(m s-1) vrel = ', best_vrel(n-1), ' +/- ', sigma_vrel(n-1)


endif ; if uncert ne 0. 

if uncert eq 0 then zkm = (height_array-r0)/1e3
if uncert ne 0 then zkm = (best_height-r0)/1e3
print, 'End of recon_traj.pro'

stop
end
